## $Id: utils.py 14589 2017-02-24 09:35:46Z henrik $
##
## Copyright (C) 2011 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
from time import time
from zope.component import createObject, getUtility
from waeup.kofa.fees import FeeTable
from waeup.kofa.interfaces import (IKofaUtils,
    ADMITTED, CLEARED, RETURNING, PAID, REGISTERED, VALIDATED)
from kofacustom.nigeria.students.utils import NigeriaStudentsUtils
from kofacustom.coewarri.interfaces import MessageFactory as _
from kofacustom.coewarri.interswitch.browser import GATEWAY_AMT


def local_nonlocal(student):
    lga = getattr(student, 'lga')
    if lga and lga.startswith('delta'):
        return 'local'
    else:
        return 'non-local'

PAYMENT_LEVELS = (100, 110, 200, 210, 300, 310, 400, 410, 500, 999)

FEES_PARAMS = (
        ('local', 'non-local'),
        ('nce_ft', 'nce_pt', 'ug_ft'),
        PAYMENT_LEVELS
    )

FEES_VALUES = (
      ( # 100      110      200      210      300      310      400      410     500  999
        (42000.0, 42000.0, 20600.0, 20600.0, 18100.0, 18100.0,     0.0,     0.0, 0.0, 0.0), # nce_ft
        (55200.0, 55200.0, 42200.0, 42200.0, 44700.0, 44700.0, 42200.0, 42200.0, 0.0, 0.0), # nce_pt
        (69200.0, 69200.0, 63100.0, 63100.0, 63100.0, 63100.0, 62100.0, 62100.0, 0.0, 0.0), # ug_ft
      ), # local
      ( #
        (54600.0, 54600.0, 22600.0, 22600.0, 20100.0, 20100.0,     0.0,     0.0, 0.0, 0.0), # nce_ft
        (55200.0, 55200.0, 42200.0, 42200.0, 44700.0, 44700.0, 42200.0, 42200.0, 0.0, 0.0), # nce_pt
        (79200.0, 79200.0, 63100.0, 63100.0, 63100.0, 63100.0, 62100.0, 62100.0, 0.0, 0.0), # ug_ft
      ), # non-local
    )

SCHOOL_FEES = FeeTable(FEES_PARAMS, FEES_VALUES)

class CustomStudentsUtils(NigeriaStudentsUtils):
    """A collection of customized methods.

    """

    def setPaymentDetails(self, category, student,
            previous_session=None, previous_level=None):
        """Create a payment ticket and set the payment data of a
        student for the payment category specified.
        """
        p_item = u''
        amount = 0.0
        p_session = student['studycourse'].current_session
        p_level = student['studycourse'].current_level
        p_current = True
        academic_session = self._getSessionConfiguration(p_session)
        if previous_session:
            return _('Previous session payment not yet implemented.'), None
        if academic_session == None:
            return _(u'Session configuration object is not available.'), None
        # Determine fee.
        if category.startswith('schoolfee'):
            try:
                certificate = student['studycourse'].certificate
                p_item = certificate.code
            except (AttributeError, TypeError):
                return _('Study course data are incomplete.'), None
            if student.state == RETURNING:
                # Override p_session and p_level
                p_session, p_level = self.getReturningData(student)
                academic_session = self._getSessionConfiguration(p_session)
                if academic_session == None:
                    return _(u'Session configuration object '
                              'is not available.'), None
            if p_level in PAYMENT_LEVELS:
                amount = SCHOOL_FEES.get_fee(
                    (
                     local_nonlocal(student),
                     student.current_mode,
                     p_level)
                    )
            if amount and category in ('schoolfee_1', 'schoolfee_2'):
                amount /= 2
            if amount:
                amount += GATEWAY_AMT
        elif category == 'clearance':
            try:
                p_item = student['studycourse'].certificate.code
            except (AttributeError, TypeError):
                return _('Study course data are incomplete.'), None
            if student.current_mode == 'nce_we_pt':
                amount = academic_session.clearance_fee_3
            elif student.current_mode == 'ug_ft':
                amount = academic_session.clearance_fee_2
            else:
                amount = academic_session.clearance_fee_1
            if local_nonlocal(student) == 'non-local':
                amount += 5000.0
        elif category == 'bed_allocation':
            p_item = self.getAccommodationDetails(student)['bt']
            amount = academic_session.booking_fee
        elif category == 'hostel_maintenance':
            amount = 0.0
            bedticket = student['accommodation'].get(
                str(student.current_session), None)
            if bedticket is not None and bedticket.bed is not None:
                p_item = bedticket.bed_coordinates
                if bedticket.bed.__parent__.maint_fee > 0:
                    amount = bedticket.bed.__parent__.maint_fee
                else:
                    # fallback
                    amount = academic_session.maint_fee
            else:
                return _(u'No bed allocated.'), None
        elif category == 'transcript':
            amount = academic_session.transcript_fee
        elif category == 'transfer':
            amount = academic_session.transfer_fee
        elif category == 'late_registration':
            amount = academic_session.late_registration_fee
        if amount in (0.0, None):
            return _('Amount could not be determined.'), None
        if self.samePaymentMade(student, category, p_item, p_session):
            return _('This type of payment has already been made.'), None
        if self._isPaymentDisabled(p_session, category, student):
            return _('This category of payments has been disabled.'), None
        payment = createObject(u'waeup.StudentOnlinePayment')
        timestamp = ("%d" % int(time()*10000))[1:]
        payment.p_id = "p%s" % timestamp
        payment.p_category = category
        payment.p_item = p_item
        payment.p_session = p_session
        payment.p_level = p_level
        payment.p_current = p_current
        payment.amount_auth = amount
        return None, payment

    # prefix
    STUDENT_ID_PREFIX = u'R'

    PORTRAIT_CHANGE_STATES = (
        ADMITTED, CLEARED, RETURNING, PAID, REGISTERED, VALIDATED)